require_relative 'subject'
require_relative 'verb'
require_relative 'object'

module Sashite
  module CGN
    module Ability
      PATTERN = /#{Subject::PATTERN}\^#{Verb::PATTERN}=#{Object::PATTERN}/

      def self.loadable? io
        !!io.match("^#{PATTERN}$")
      end

      def self.load io
        raise ArgumentError unless loadable? io

        subject = Subject.load io.split('^').fetch 0
        verb    = Verb.load io.split('^').fetch(1).split('=').fetch 0
        object  = Object.load io.split('=').fetch 1

        {
          subject: subject,
          verb: verb,
          object: object
        }
      end

      def self.dumpable? io
        io.is_a?(Hash) &&
        io.keys.sort == %i(object subject verb) &&
        Subject.dumpable?(io.fetch(:subject)) &&
        Verb.dumpable?(io.fetch(:verb)) &&
        Object.dumpable?(io.fetch(:object))
      end

      def self.dump io
        raise ArgumentError unless dumpable? io

        Subject.dump(io.fetch(:subject)) + '^' +
        Verb.dump(io.fetch(:verb)) + '=' +
        Object.dump(io.fetch(:object))
      end
    end
  end
end
