require_relative 'actor'
require_relative 'gameplay_into_base64'

module Sashite
  module CGN
    module PromotableIntoActors
      PATTERN = /(#{GameplayIntoBase64::PATTERN},)*#{Actor::PATTERN}/

      def self.loadable? io
        io.match("^#{PATTERN}$") &&
        io.split(',').uniq.join(',') == io
      end

      def self.load io
        raise ArgumentError unless loadable? io

        io.split(',').map do |value|
          Actor.loadable?(value) ? Actor.load(value) : GameplayIntoBase64.load(value)
        end
      end

      def self.dumpable? io
        io.is_a?(Array) &&
        io.length == io.uniq.length &&
        io[0..-2].all? {|i| GameplayIntoBase64.dumpable? i } &&
        Actor.dumpable?(io.last)
      end

      def self.dump io
        raise ArgumentError unless dumpable? io

        actors = io[0..-2].map {|a| GameplayIntoBase64.dump a }
        actors << Actor.dump(io.last)
        actors.join(',')
      end
    end
  end
end
