require_relative 'attacked'
require_relative 'occupied'
require_relative 'area'

module Sashite
  module CGN
    module Square
      PATTERN = /#{Attacked::PATTERN}@#{Occupied::PATTERN}\+#{Area::PATTERN}/

      def self.loadable? io
        !!io.match("^#{PATTERN}$")
      end

      def self.load io
        raise ArgumentError unless loadable? io

        attacked = Attacked.load io.split('@').fetch(0)
        occupied = Occupied.load io.split('@').fetch(1).split('+').fetch(0)
        area = Area.load io.split('+').fetch(1)

        {
          :"...attacked?" => attacked,
          :"...occupied!" => occupied,
          :"area" => area
        }
      end

      def self.dumpable? io
        io.is_a?(Hash) &&
        io.keys.sort == %i(...attacked? ...occupied! area) &&
        Attacked.dumpable?(io.fetch(:'...attacked?')) &&
        Occupied.dumpable?(io.fetch(:'...occupied!')) &&
        Area.dumpable?(io.fetch(:'area'))
      end

      def self.dump io
        raise ArgumentError unless dumpable? io

        Attacked.dump(io.fetch(:'...attacked?')) + '@' +
        Occupied.dump(io.fetch(:'...occupied!')) + '+' +
        Area.dump(io.fetch(:area))
      end
    end
  end
end
