require_relative 'ally'
require_relative 'actor'
require_relative 'state'

module Sashite
  module CGN
    module Subject
      PATTERN = /#{Ally::PATTERN}<#{Actor::PATTERN}>#{State::PATTERN}/

      def self.loadable? io
        !!io.match("^#{PATTERN}$")
      end

      def self.load io
        raise ArgumentError unless loadable? io

        ally = Ally.load io.split('<').fetch(0)
        actor = Actor.load io.split('<').fetch(1).split('>').fetch(0)
        state = State.load io.split('>').fetch(1)

        {
          :"...ally?" => ally,
          actor: actor,
          state: state
        }
      end

      def self.dumpable? io
        io.is_a?(Hash) &&
        io.keys.sort == %i(...ally? actor state) &&
        Ally.dumpable?(io.fetch(:'...ally?')) &&
        Actor.dumpable?(io.fetch(:actor)) &&
        State.dumpable?(io.fetch(:state))
      end

      def self.dump io
        raise ArgumentError unless dumpable? io

        Ally.dump(io.fetch(:'...ally?')) + '<' +
        Actor.dump(io.fetch(:actor)) + '>' +
        State.dump(io.fetch(:state))
      end
    end
  end
end
