require_relative 'direction'
require_relative 'maximum_magnitude'
require_relative 'name'

module Sashite
  module CGN
    module Verb
      PATTERN = /#{Name::PATTERN}\[#{Direction::PATTERN}\]#{MaximumMagnitude::PATTERN}/

      def self.loadable? io
        !!io.match("^#{PATTERN}$")
      end

      def self.load io
        raise ArgumentError unless loadable? io

        name = Name.load io.split('[').fetch 0
        direction = Direction.load io.split('[').fetch(1).split(']').fetch 0
        maximum_magnitude = MaximumMagnitude.load io.split(']').fetch(1).split('/').fetch 0

        {
          name: name,
          vector: {
            :"...maximum_magnitude" => maximum_magnitude,
            direction: direction
          }
        }
      end

      def self.dumpable? io
        io.is_a?(Hash) &&
        io.keys.sort == %i(name vector) &&
        io.fetch(:vector).is_a?(Hash) &&
        io.fetch(:vector).keys.sort == %i(...maximum_magnitude direction) &&
        MaximumMagnitude.dumpable?(io.fetch(:vector).fetch(:"...maximum_magnitude")) &&
        Direction.dumpable?(io.fetch(:vector).fetch(:direction))
      end

      def self.dump io
        raise ArgumentError unless dumpable? io

        Name.dump(io.fetch(:name)) + '[' +
        Direction.dump(io.fetch(:vector).fetch(:direction)) + ']' +
        MaximumMagnitude.dump(io.fetch(:vector).fetch(:'...maximum_magnitude'))
      end
    end
  end
end
